from setuptools import setup

version = '0.2.5'

setup(name='VisualAssertLibrary',
      version=version,
      description='Robot Framework Visual Assert Library',
      packages=['VisualAssertLibrary'],
      author='Dmytro Bondarchuk',
      author_email='d.bondarchuk.spv@gmail.com',
      zip_safe=False)
